/**
 * \file application.h
 * QApplication with settings.
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 17 May 2010
 *
 * Copyright (C) 2010  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPLICATION_H
#define APPLICATION_H

#include <QApplication>
#include <QStringList>
#include <QVector>
#include <QFont>
#include <QMap>
#include "taggingscheme.h"
#include "applicationpaths.h"
#include "coverimage.h"

#define App (static_cast<Application *>(QCoreApplication::instance()))

class IQoobarPlugin;
class IDownloadPlugin;

class Application : public QApplication
{
    Q_OBJECT
public:
    explicit Application(int & argc, char ** argv, bool useGui=true);
    ~Application();
    void writeResources();
    void loadTranslations();
    void setId3v1Encoding(const QString &s);
    QStringList readResource(const QString &fileName);
    void updateCompletions(const int tagID, const QStringList &collected);
    void updateCompletion(const int tagID, const QString &collected);
    void addRenamePattern(const QString &s);
    void addFillPattern(const QString &s);
    void setFilesNames(const QStringList names) {filesNames = names;}

    void loadPlugins();
#ifdef HAVE_QT5
    QList<QJsonObject> downloadPlugins;
    QList<QJsonObject> plugins;
#else
    QMap<QString, IDownloadPlugin *> downloadPlugins;
    QMap<QString, IQoobarPlugin *> plugins;
#endif
    void readGuiSettings();
    void readGlobalSettings();
    void clearSettings();
    void resetSettings();
    void writeGuiSettings();
    void writeGlobalSettings();

    bool consoleMode;
    QString langID; //language id of the app language
    QString chars; //characters with diacritics
    QString lastDirectory;  //last added directory
    QString lastCueFile;
    bool mp3readape;
    bool mp3writeape;
    bool mp3readid3;
    bool mp3writeid3;
    QString player;
    QByteArray columns160; //horizontal header state
    QByteArray rows; //vertical header state
    bool saveChanges; //whether or not save all changes when closing app
    bool trim; //whether or not trim tags when saving files
    QStringList patterns;
    bool autoexpand;

    bool oggPictureNew;

    QVector<int> tagStatus;

    QString freedbHello;

    bool useProxy;
    QString proxyServer;
    QString proxyPort;
    QString proxyLogin;
    QString proxyPassword;

    //rename options
    QString destinationFolder;
    bool replaceWinChars;
    QString winCharsReplacer;
    bool replaceSpaces;
    QString spacesReplacer;
    bool removeDiacritics;
    int ccase;
    bool changeCase;
    int maximumLength;
    bool trimFileLength;
    int renamingOperation;
    bool removeFolder;
    bool applyToFolders;

    //autocompletion options
    bool completionStyle;
    bool collectSilently;
    QVector<bool> useCompletion;
    QHash<int,QStringList> completions;


    CoverImage imageBuffer;

    bool charsShown;
    QFont charsFont;

    QString cdromDevice;

    int id3v1Synchro;
    bool id3v1Transliterate;
    QString id3v1Encoding;

    QByteArray splitterState;
    QByteArray dirSplitterState;
    bool showDirView;


    QString currentSchemeName;
    TaggingScheme *currentScheme;

    bool useUndo;
//    bool useDelayedUpdate;

    QString encaGuessLanguage;
    QByteArray geometry;
    bool writeFieldsSeparately;
    ApplicationPaths *paths;
    bool copyFiles;
    QString discidLibraryPath;

    QStringList fillPatterns;
    int id3v2version;

    QStringList filesNames;

    bool verbose;
    bool recursive;

    bool mpcWriteRg;
private:
    QTranslator *appTranslator;
    QTranslator *qtTranslator;
};

#endif // APPLICATION_H
